@@ -2,6 +2,7 @@
 Copyright © 2021 yu9824
 '''
 
+from collections import Counter
 import numpy as np
 import pandas as pd
 import re
@@ -11,21 +12,44 @@ from copy import copy
 from math import ceil, floor
 
 
-default_elements = ("H", "He", "Li", "Be", "B", "C", "N", "O", "F", "Ne", "Na", "Mg", "Al", "Si", "P", "S", "Cl", "Ar", "K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn", "Ga", "Ge", "As", "Se", "Br", "Kr", "Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd", "In", "Sn", "Sb", "Te", "I", "Xe", "Cs", "Ba", "La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb", "Dy", "Ho", "Er", "Tm", "Yb", "Lu", "Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg", "Tl", "Pb", "Bi", "Po", "At", "Rn", "Fr", "Ra", "Ac", "Th", "Pa", "U", "Np", "Pu", "Am", "Cm", "Bk", "Cf", "Es", "Fm", "Md", "No", "Lr", "Rf", "Db", "Sg", "Bh", "Hs", "Mt", "Ds", "Rg", "Cn", "Nh", "Fl", "Mc", "Lv", "Ts", "Og")
+DEFAULT_ELEMENTS = ("H", "He", "Li", "Be", "B", "C", "N", "O", "F", "Ne", "Na", "Mg", "Al", "Si", "P", "S", "Cl", "Ar", "K", "Ca", "Sc", "Ti", "V", "Cr", "Mn", "Fe", "Co", "Ni", "Cu", "Zn", "Ga", "Ge", "As", "Se", "Br", "Kr", "Rb", "Sr", "Y", "Zr", "Nb", "Mo", "Tc", "Ru", "Rh", "Pd", "Ag", "Cd", "In", "Sn", "Sb", "Te", "I", "Xe", "Cs", "Ba", "La", "Ce", "Pr", "Nd", "Pm", "Sm", "Eu", "Gd", "Tb", "Dy", "Ho", "Er", "Tm", "Yb", "Lu", "Hf", "Ta", "W", "Re", "Os", "Ir", "Pt", "Au", "Hg", "Tl", "Pb", "Bi", "Po", "At", "Rn", "Fr", "Ra", "Ac", "Th", "Pa", "U", "Np", "Pu", "Am", "Cm", "Bk", "Cf", "Es", "Fm", "Md", "No", "Lr", "Rf", "Db", "Sg", "Bh", "Hs", "Mt", "Ds", "Rg", "Cn", "Nh", "Fl", "Mc", "Lv", "Ts", "Og")
 # --- tools ---
+def _check_value(x:str) -> float:
+    """check value of the elements
+
+    Parameters
+    ----------
+    x : str
+
+    Examples
+    ----------
+    >>> _check_value('1.2')
+    1.2
+    >>> _check_value('1/2')
+    0.5
+
+    Returns
+    -------
+    float
+        checked value
+    """
+    c = Counter(x)
+    if '/' in c and c['/'] == 1:
+        _a, _b = map(float, x.split('/'))
+        return _a / _b
+    else:
+        return float(x)
+
 def _flatten_and_trim(x):
     # とにかく一次元のリスト (or np.ndarray) 化
-    if type(x) == np.ndarray:
-        y = x.flatten()
-    elif type(x) == str:
-        y = [x]
-    else:
-        y = np.array(x).flatten()
+    if isinstance(x, str):
+        x = [x]
+    _x = np.array(x).ravel()
     # 空白を削除してリストとして統一
-    return list(map(lambda x:x.replace(' ',''), y))
+    return list(map(lambda x:x.replace(' ',''), _x))
 
 # --- main function ---
-def element_recognition(compositions, elements = None):
+def element_recognition(compositions, elements = None) -> pd.DataFrame:
     '''
     Parameters
     ----------
@@ -40,7 +64,7 @@ def element_recognition(compositions, elements = None):
     '''
 
     if elements is None:
-        elements = default_elements
+        elements = DEFAULT_ELEMENTS
 
     lst_compositions = _flatten_and_trim(compositions)
 
@@ -56,7 +80,7 @@ def element_recognition(compositions, elements = None):
             polyatomic_ion.append(ele)
 
 
-    ratio_ = np.zeros([len(lst_compositions), len(elements)])
+    ratio_ = np.zeros([len(lst_compositions), len(elements)], dtype=float)
     for i, comp in enumerate(lst_compositions):
         del_index = [j for j, x in enumerate(comp) if x.isupper() or x in delimiter]    # 大文字や()のときで区切り文字．
 
@@ -65,14 +89,21 @@ def element_recognition(compositions, elements = None):
             del_index = [j for j in del_index if j <= k or k + len(poly) <= j]
 
         dict_comp = {}  # key: index(元素の位置), value: value(組成数), symbol(元素記号, 多原子イオンも含む)
-        dict_brackets = {'f':[], 'b':[], 'value':[]}    # かっこの開始位置 (f)， 終了位置 (b), かっこ内の元素を何倍するか (value) を取っておく辞書
+        dict_brackets = {
+            'f':[],     # かっこの開始位置
+            'b':[],     # かっこの終了位置
+            'value':[]  # かっこ内の元素を何倍するか
+        }
         for j in range(len(del_index)): #'(', ')', 'polyatomic_ion', 'monatomic_ion'がそれぞれどこか記録．
             # aとして抜き出す部分を取り出す．
             a = comp[del_index[j]:] if j == len(del_index) - 1 else comp[del_index[j]:del_index[j+1]]
 
             # 取り出したaを数値なのか，()なのかなど認識し，数字と元素に分けるなどする．
             if a in polyatomic_ion: #'NH4'(polyatomic_ion)
-                dict_comp[del_index[j]] = {'value':1.0, 'symbol':a}
+                dict_comp[del_index[j]] = {
+                    'value':1.0,
+                    'symbol':a,
+                }
             elif a == delimiter[0]: #'('
                 dict_brackets['f'].append(del_index[j])
             elif a == delimiter[1]: #')'
@@ -80,14 +111,20 @@ def element_recognition(compositions, elements = None):
                 dict_brackets['value'].append(1.0)
             elif delimiter[1] in a: #')n'
                 dict_brackets['b'].append(del_index[j])
-                dict_brackets['value'].append(float(a.strip(delimiter[1])))
+                dict_brackets['value'].append(_check_value(a.strip(delimiter[1])))
             else:   #monatomic_ion
                 for k, s in enumerate(a):
                     if not s.isalpha():
-                        dict_comp[del_index[j]] = {'value':float(a[k:]), 'symbol':a[:k]}
+                        dict_comp[del_index[j]] = {
+                            'value':_check_value(a[k:]),
+                            'symbol':a[:k],
+                        }
                         break
-                else:   #最後までs.isalpha() == Falseとならなかった　(数字が現れなかった) 場合．
-                    dict_comp[del_index[j]] = {'value':1.0, 'symbol':a}
+                else:   #最後までs.isalpha() == Falseとならなかった (数字が現れなかった) 場合．
+                    dict_comp[del_index[j]] = {
+                        'value':1.0,
+                        'symbol':a,
+                    }
 
         #()のペアを一致させるための処理
         dict_brackets['f'].reverse()
@@ -114,7 +151,7 @@ def element_recognition(compositions, elements = None):
     return df_output
 
 
-def get_ratio(products, materials, match_all = True, elements = None):
+def get_ratio(products, materials, exact = True, elements = None) -> pd.DataFrame:
     '''
     Parameters
     ----------
@@ -124,7 +161,7 @@ def get_ratio(products, materials, match_all = True, elements = None):
     materials: list
         原料．リスト．無駄な原料を入れると計算できなくなることが多いので入れるべきではない．(必須)
     
-    mathch_all: bool, default True
+    exact: bool, default True
         検算してすべての元素の割合が合ってるかを確かめ，一つでも元素の数が合わないとき，Noneとして返す．
 
     elements: list, default None.
@@ -137,11 +174,11 @@ def get_ratio(products, materials, match_all = True, elements = None):
 
     Examples
     ----------
-    >>> materials = ['Li2O', 'LaO3', 'TiO2']
-    >>> products = ['Li2LaTiO6']
+    >>> materials = ['Li2O', 'La2O3', 'TiO2']
+    >>> products = ['Li2La2TiO6']
     >>> get_ratio(products, materials)
-               Li2O  LaO3  TiO2
-    Li2LaTiO6   1.0   1.0   1.0
+                Li2O  La2O3  TiO2
+    Li2La2TiO6   1.0    1.0   1.0
     '''
     products = _flatten_and_trim(products)
     materials = _flatten_and_trim(materials)
@@ -174,21 +211,10 @@ def get_ratio(products, materials, match_all = True, elements = None):
             print(materials_nonzero, '\n', products_nonzero)
             raise ValueError("We can't solve.\nWe can't get square matrix.")
     else:
-        df_output = pd.DataFrame()
-        for name, series in products_nonzero.iterrows():
-            matrix = materials_nonzero.to_numpy().transpose() / series.to_numpy()
-            boolean = np.all(matrix, axis = 1)
-            if np.sum(boolean) != 0:
-                sr_x = pd.Series([1 / matrix[boolean].flatten()[0] if boo else 0 for boo in boolean], index = materials_nonzero.columns, name = name)
-                df_output = pd.concat([df_output, sr_x], axis = 1, sort = False)
-                continue
-            else:
-                print(materials_nonzero, '\n', products_nonzero)
-                raise ValueError("We can't solve.\nThe rank(A) is lower than a number of variables(materials).")
-        else:
-            return df_output.transpose()
+        raise ValueError("We can't solve.\nThe rank(A) is lower than a number of variables(materials).")
 
     df_output = pd.DataFrame()
+    _srs = []
     for name, series in products_nonzero.iterrows():
         b = np.delete(series.to_numpy(), del_index)
         try:
@@ -196,25 +222,29 @@ def get_ratio(products, materials, match_all = True, elements = None):
         except np.linalg.LinAlgError:
             raise np.linalg.LinAlgError("We can't solve.\n", A, b)
         sr_x = pd.Series(x, index = materials_nonzero.columns, name = name)
-        if match_all:
+        if exact:
             ar_memo = np.zeros(df_products.shape[1])
             for y, z in zip(x, df_materials.to_numpy()):
                 ar_memo += y * z
             if np.allclose(ar_memo, df_products.loc[name].to_numpy()):    # すべての元素が検算で正しいとされるならば
-                df_output = pd.concat([df_output, sr_x], axis = 1, sort = False)
+                # df_output = pd.concat([df_output, sr_x], axis = 1, sort = False)
+                _srs.append(sr_x)
             else:
-                df_output = pd.concat([df_output, pd.Series([None] * len(materials_nonzero.columns), index = materials_nonzero.columns, name = name)], axis = 1, sort = False)
+                _srs.append(pd.Series([None] * len(materials_nonzero.columns), index = materials_nonzero.columns, name = name))
+                # df_output = pd.concat([df_output, pd.Series([None] * len(materials_nonzero.columns), index = materials_nonzero.columns, name = name)], axis = 1, sort = False)
         else:
-            df_output = pd.concat([df_output, sr_x], axis = 1, sort = False)
-    return df_output.transpose()
+            _srs.append(sr_x)
+            # df_output = pd.concat([df_output, sr_x], axis = 1, sort = False)
+    df_output = pd.concat(_srs, axis=1).transpose()
+    return df_output
 
 
-def make_compositions(materials, ratio = None, easy = True, max_comp = 15, front = None, back = None, max_show_prec = 3, elements = None):
+def make_compositions(materials, ratio = None, easy = True, max_comp:int = 15, front = None, back = None, max_show_prec:int = 3, elements = None) -> pd.DataFrame:
     '''
     Parameters
     ----------
     materials : list
-        e.g.) ['Li2O', 'LaO3', 'TiO2']
+        e.g.) ['Li2O', 'La2O3', 'TiO2']
 
     ratio: 2D-list (pd.DataFrame, np.ndarray etc.), default None
         If no input is given, an appropriate composition is generated.
@@ -232,7 +262,7 @@ def make_compositions(materials, ratio = None, easy = True, max_comp = 15, front
         An element that is preferentially placed behind a composition when it is generated. The more elements are in front, the more elements are in the back. If duplicate elements are specified, back is given priority.
 
     max_show_prec: int, default 3
-        The maximum number of decimal places to display when creating a compound.
+        The maximum number of float places to display when creating a compound.
 
     elements: list, default None.
         This is variable for element_recognition function.
@@ -257,8 +287,9 @@ def make_compositions(materials, ratio = None, easy = True, max_comp = 15, front
         front = [ele for ele in front if ele not in back]
 
     # 何も入力されなければ自動生成
+    p = product(np.arange(max_comp+1), repeat = len(materials))
     if ratio is None:
-        ratio = [i for i in product(np.arange(max_comp), repeat = len(materials)) if sum(i) == max_comp] if easy else [i for i in product(np.arange(max_comp), repeat = len(materials)) if sum(i) != 0]
+        ratio = [i for i in p if sum(i) == max_comp] if easy else [i for i in p if sum(i) != 0]
     ratio = np.array(ratio)
 
     if len(ratio.shape) == 1 and ratio.shape[0] == len(materials):
@@ -319,26 +350,26 @@ def make_compositions(materials, ratio = None, easy = True, max_comp = 15, front
 
 
 if __name__ == '__main__':
-    from pdb import set_trace
+    # from pdb import set_trace
     # pd.set_option('display.max_columns', 150)
 
     # products = ['LaO3', 'Li2LaO']
     # # products = 'Li0.33La0.5TiO3'
     # products = ['Li2LaTiO6', 'Li0.33La0.5TiO3', 'Li2LaTiO6']
     # # products = ['Li2LaTiO6', 'Li0.33La0.5TiO3', 'Li2LaTiO6', 'Li2O']
-    materials = ['Li2O', 'LaO3', 'TiO2']
+    # materials = ['Li2O', 'LaO3', 'TiO2']
 
     # # products = 'LiLaO2'
     # # products = ['Li2La2O4', 'LiLaO2']
     # # materials = ['Li2TiO3', 'Li2BaO3', 'LiLaO2']
 
     # df_er = element_recognition(products)
-    # df_r = Ratio(products, materials, match_all = True)
+    # df_r = Ratio(products, materials, exact = True)
     # print(df_r)
     # # print(df_er, df_r)
 
-    df_m = make_compositions(materials, ratio = [2/3, 2/3, 3])
-    print(df_m)
+    # df_m = make_compositions(materials, ratio = [2/3, 2/3, 3])
+    # print(df_m)
     # # print(df_m)
-    # from doctest import testmod
-    # testmod(verbose=True)
+    from doctest import testmod
+    testmod(verbose=True)
